<?php
//
// Nagios XI Operations Center Component
// Copyright (c) 2011-2018 Nagios Enterprises, LLC. All rights reserved.
//

require_once(dirname(__FILE__) . '/../../common.inc.php');

// Initialization and session start
pre_init();
init_session();
grab_request_vars();
check_prereqs();

// Check authentication
check_authentication(false);

$host = grab_request_var("host", "");
$service = grab_request_var("service", "");
$hostgroup = grab_request_var("hostgroup", "");
$servicegroup = grab_request_var("servicegroup", "");
$state = grab_request_var("state", "");
$host_orderby = grab_request_var("host_orderby", "");
$service_orderby = grab_request_var("service_orderby", "");

?>

<?php
// Grab extra filters
$filters = grab_request_var('filters', '');

// If the hide_not_dis is in the array, the user has turned that filter on
if (in_array('hide_not_dis', $filters)) {
    // Turn filter on
    set_user_meta(0, 'nocscreen_hide_not_disabled', 1, false);
} else {
    set_user_meta(0, 'nocscreen_hide_not_disabled', 0, false);
}

// If the hide_handled is in the array, the user has turned that filter on
if (in_array('hide_handled', $filters)) {
    // Turn filter on
    set_user_meta(0, 'nocscreen_hide_handled', 1, false);
} else {
    set_user_meta(0, 'nocscreen_hide_handled', 0, false);
}

// If the hide_soft is in the array, the user has turned that filter on
if (in_array('hide_soft', $filters)) {
    // Turn filter on
    set_user_meta(0,'nocscreen_hide_soft_states', 1, false);
} else {
    set_user_meta(0,'nocscreen_hide_soft_states', 0, false);
}

if (!empty($host_orderby)){
    set_user_meta(0,'nocscreen_host_orderby', $host_orderby);
}

if (!empty($service_orderby)){
    set_user_meta(0,'nocscreen_service_orderby', $service_orderby);
}

// Grab user metas after they have been updated
// ** There is probably a better way to do this **
$hide_not_dis = get_user_meta(0, 'nocscreen_hide_not_disabled');
$hide_handled = get_user_meta(0, 'nocscreen_hide_handled');
$hide_soft = get_user_meta(0, 'nocscreen_hide_soft_states');
$host_orderby = grab_request_var("host_orderby", get_user_meta(0, 'nocscreen_host_orderby', "last_state_change:a"));
$service_orderby = grab_request_var("service_orderby", get_user_meta(0, 'nocscreen_service_orderby', "last_state_change:a"));

$title = _("Operations Center");
do_page_start(array("page_title" => $title), true);
?>

<!-- Include sumoselect -->
<script type="text/javascript" src="jquery.sumoselect.js"></script>
<link rel="stylesheet" type="text/css" href="sumoselect.css">

<style type="text/css">
    #topbar { margin-top: 10px; vertical-align: top; }
    h1 { margin: 0; padding: 0; }
    h1 img { margin-right: 10px; }
    #leftside { float: left; }
    #rightside { float: right; }
    .strong { font-weight: bold; }
    #lastUpdate { color: #777; font-size: 11px; margin: 10px 0; }
    div.duration { width: 125px; }
    #noc_filter{ margin: 0; }
    .noc-filters { margin: 5px 0 15px 0; }
    .filters-btn { margin-left: 10px; }
</style>

<script type="text/javascript">
REFRESH_RATE = 30; // Number of seconds between NOC refreshes

$(document).ready(function () {

    // Initialize our sumoselects
    // These are all done independently so we can act upon them more easily
    extraFilterSelect = $('#extra-filters-select').SumoSelect({
        okCancelInMulti: true,
        captionFormatAllSelected: "<?php echo _('All filters selected'); ?>",
        csvDispCount: 2,
        placeholder: '<?php echo _('Select Additional Filters'); ?>'
    });

    stateSelect = $('#state-select').SumoSelect({});

    hostSelect = $('#host-select').SumoSelect({
        triggerChangeCombined: true,
        csvDispCount: 2,
        search: true,
        searchText: 'Search...'
    });

    serviceSelect = $('#service-select').SumoSelect({
        triggerChangeCombined: true,
        csvDispCount: 2,
        search: true,
        searchText: 'Search...'
    });

    servicegroupSelect = $('#servicegroup-select').SumoSelect({
        triggerChangeCombined: true,
        csvDispCount: 2,
        search: true,
        searchText: 'Search...'
    });

    hostgroupSelect = $('#hostgroup-select').SumoSelect({
        triggerChangeCombined: true,
        csvDispCount: 2,
        search: true,
        searchText: 'Search...'
    });

    // Main NOC content and loop
    noc_load_content();

    // Hide/show filters
    $('.filters-btn').click(function() {
        if ($('.noc-filters').is(':visible')) {
            $('.noc-filters').hide();
            $('.filters-btn').html('<i class="fa fa-filter"></i> <?php echo _("Show Filters"); ?>');
        } else {
            $('.noc-filters').show();
            $('.filters-btn').html('<i class="fa fa-filter"></i> <?php echo _("Hide Filters"); ?>');
        }
    });

    // Clear hostgroup and servicegroup selects if a host is selected
    $(hostSelect).change(function ()
    {
        // Check for empty string to avoid infinite loop
        if ($(this).val() !== '') {
            // Clear the hostgroup select
            $('#hostgroup-select > .placeholder').prop('disabled', false);
            hostgroupSelect.sumo.selectItem('');
            hostgroupSelect.sumo.unSelectAll();
            $('#hostgroup-select > .placeholder').prop('disabled', true);
            hostgroupSelect.sumo.reload();

            // Clear the servicegroup
            $('#servicegroup-select > .placeholder').prop('disabled', false);
            servicegroupSelect.sumo.selectItem('');
            servicegroupSelect.sumo.unSelectAll();
            $('#servicegroup-select > .placeholder').prop('disabled', true);
            servicegroupSelect.sumo.reload();
        } else {
            return;
        }
    });

    // Clear host and servicegroup selects if a hostgroup is selected
    $(hostgroupSelect).change(function ()
    {
        // Check for empty string to avoid infinite loop
        if ($(this).val() !== '') {
            // Clear the host select
            $('#host-select > .placeholder').prop('disabled', false);
            hostSelect.sumo.selectItem('');
            hostSelect.sumo.unSelectAll();
            $('#host-select > .placeholder').prop('disabled', true);
            hostSelect.sumo.reload();

            // Clear the servicegroup
            $('#servicegroup-select > .placeholder').prop('disabled', false);
            servicegroupSelect.sumo.selectItem('');
            servicegroupSelect.sumo.unSelectAll();
            $('#servicegroup-select > .placeholder').prop('disabled', true);
            servicegroupSelect.sumo.reload();
        } else {
            return false;
        }
        
    });

    // Clear host and servicegroup selects if a hostgroup is selected
    $(servicegroupSelect).change(function ()
    {
        // Check for empty string to avoid infinite loop
        if ($(this).val() !== '') {
            // Clear the host select
            $('#host-select > .placeholder').prop('disabled', false);
            hostSelect.sumo.selectItem('');
            hostSelect.sumo.unSelectAll();
            $('#host-select > .placeholder').prop('disabled', true);
            hostSelect.sumo.reload();

            // Clear the servicegroup
            $('#hostgroup-select > .placeholder').prop('disabled', false);
            hostgroupSelect.sumo.selectItem('');
            hostgroupSelect.sumo.unSelectAll();
            $('#hostgroup-select > .placeholder').prop('disabled', true);
            hostgroupSelect.sumo.reload();
        } else {
            return false;
        }

    });
    
    $('#hosttable .host_order').live('click', function() {

        if($('#host_orderby').val() == 'name:a')
            $('#host_orderby').val('name:d');
        else
            $('#host_orderby').val('name:a');
        noc_load_content();
    });
    
    $('#hosttable .duration_order').live('click', function() {

        if($('#host_orderby').val() == 'last_state_change:a')
            $('#host_orderby').val('last_state_change:d');
        else
            $('#host_orderby').val('last_state_change:a');
        noc_load_content();
    });
    
    $('#servicetable .host_order').live('click', function() {

        if($('#service_orderby').val() == 'host_name:a')
            $('#service_orderby').val('host_name:d');
        else
            $('#service_orderby').val('host_name:a');
        noc_load_content();
    });
    
    $('#servicetable .service_order').live('click', function() {

        if($('#service_orderby').val() == 'service_description:a')
            $('#service_orderby').val('service_description:d');
        else
            $('#service_orderby').val('service_description:a');
        noc_load_content();
    });
    
    $('#servicetable .duration_order').live('click', function() {

        if($('#service_orderby').val() == 'last_state_change:a')
            $('#service_orderby').val('last_state_change:d');
        else
            $('#service_orderby').val('last_state_change:a');
        noc_load_content();
    });

});

function noc_load_content()
{
    var datastring = $("#noc_filter").serialize();

    // Main NOC content
    $.ajax({
        url: 'nocscreenapi.php',
        data: datastring
    }).done(function (html) {
        var d = new Date();
        $('#lastUpdate').empty();
        $('#lastUpdate').append(('Last Update: ' + d.toString() ));
        $('#content').html(html);
    });

    // Summary bar info
    $.ajax({
        url: 'nocscreenapi.php?summary=true',
        data: datastring
    }).done(function (html) {
        $('#rightside').html(html);
    });

    setTimeout(noc_load_content, (REFRESH_RATE * 1000));
}
</script>

<div id="topbar">
    <div id="leftside">
        <h1>
            <img src="/nagiosxi/images/nagiosxi-logo-small.png" alt="<?php echo _("Nagios XI"); ?>">
            <?php echo _("Operations Center"); ?>
            <button type="button" class="filters-btn btn btn-xs btn-default"><i class="fa fa-filter"></i> <?php echo _("Hide Filters"); ?></button>
        </h1>
        <div id="lastUpdate"></div>
    </div>
    <div id="rightside"></div>
    <div class="clear"></div>
</div>

<div class="noc-filters">    
    <form id="noc_filter" method="get" action="<?php echo htmlentities($_SERVER["REQUEST_URI"]); ?>">

        <div class="input-group" style="width: 150px; display: inline-table; vertical-align: bottom;">
            <label class="input-group-addon"><?php echo _('Host'); ?></label>
            <select class="hide" name="host" id="host-select">
                <option class="placeholder" value="" selected disabled><?php echo _('Select a Host'); ?></option>
                <option value=""><?php echo _('Show all Hosts'); ?></option>
                <?php
                $args = array('brevity' => 1, 'orderby' => 'host_name:a');
                $oxml = get_xml_host_objects($args);
                if ($oxml) {
                    foreach ($oxml->host as $hostobject) {
                        $name = strval($hostobject->host_name);
                        echo "<option value='" . $name . "' " . is_selected($host, $name) . ">$name</option>\n";
                    }
                }
                ?>
            </select>
        </div>
 
        <div class="input-group" style="width: 150px; display: inline-table; vertical-align: bottom;">
            <label class="input-group-addon"><?php echo _('Hostgroup'); ?></label>
            <select class="hide" name="hostgroup" id="hostgroup-select">
                <option class="placeholder" value="" selected disabled><?php echo _('Select a Hostgroup'); ?></option>
                <option value=""><?php echo _('Show all Hostgroups'); ?></option>
                <?php
                $args = array('orderby' => 'hostgroup_name:a');
                $oxml = get_xml_hostgroup_objects($args);
                if ($oxml) {
                    foreach ($oxml->hostgroup as $hg) {
                        $name = strval($hg->hostgroup_name);
                        echo "<option value='" . $name . "' " . is_selected($hostgroup, $name) . ">$name</option>\n";
                    }
                }
                ?>
            </select>
        </div>

        <div class="input-group" style="width: 150px; display: inline-table; vertical-align: bottom;">
            <label class="input-group-addon"><?php echo _('Servicegroup'); ?></label>
            <select class="hide" name="servicegroup" id="servicegroup-select">
                <option class="placeholder" value="" selected disabled><?php echo _('Select a Servicegroup'); ?></option>
                <option value=""><?php echo _('Show all Servicegroups'); ?></option>
                <?php
                $args = array('orderby' => 'servicegroup_name:a');
                $oxml = get_xml_servicegroup_objects($args);
                if ($oxml) {
                    foreach ($oxml->servicegroup as $sg) {
                        $name = strval($sg->servicegroup_name);
                        echo "<option value='" . $name . "' " . is_selected($servicegroup, $name) . ">$name</option>\n";
                    }
                }
                ?>
            </select>
        </div>

        <div class="input-group" style="width: 200px; display: inline-table; vertical-align: bottom;">
            <label class="input-group-addon"><?php echo _('Service State'); ?></label>
            <select class="hide" name="state" id="state-select">
                <option class="placeholder" selected disabled><?php echo _('Select a Service State'); ?></option>
                <option><?php echo _('Show all states'); ?></option>
                <option value="1" <?php echo is_selected($state, 1); ?> ><?php echo _('Warning') ?></option>
                <option value="2" <?php echo is_selected($state, 2); ?> ><?php echo _('Critical') ?></option>
                <option value="3" <?php echo is_selected($state, 3); ?> ><?php echo _('Unknown') ?></option>
            </select>
        </div>

        <div class="input-group" style="width: 320px; display: inline-table; vertical-align: bottom;">
            <label class="input-group-addon"><?php echo _('Additional Filters'); ?></label>
            <select class="hide" name="filters[]" id="extra-filters-select" multiple>
                <option value="hide_not_dis" <?php echo is_selected($hide_not_dis, 1); ?> ><?php echo _('Hide notifications disabled') ?></option>
                <option value="hide_handled" <?php echo is_selected($hide_handled, 1); ?> ><?php echo _('Hide handled') ?></option>
                <option value="hide_soft" <?php echo is_selected($hide_soft, 1); ?> ><?php echo _('Hide soft states') ?></option>
            </select>
        </div>

        <input id="host_orderby" type="hidden" name="host_orderby" value="<?php echo $host_orderby; ?>">
        <input id="service_orderby" type="hidden" name="service_orderby" value="<?php echo $service_orderby; ?>">
        <button type="submit" class="reporttimesubmitbutton btn btn-sm btn-primary" style="vertical-align: top;" name="reporttimesubmitbutton"><?php echo _("Update"); ?></button>

    </form>
    <div class="clear"></div>
</div>

<div id="content"></div>

<?php
do_page_end(true);
